import click
from pywhat import identifier
from pywhat import printer


@click.command()
@click.argument("text_input", required=True)
def main(text_input):
    """
    What - Identify what something is.\n
    Made by Bee https://twitter.com/bee_sec_san
    https://github.com/bee-san\n

    Examples:

        * what "HTB{this is a flag}"

        * what "0x52908400098527886E0F7030069857D2E4169EE7"


    """

    what_obj = What_Object()
    identified_output = what_obj.what_is_this(text_input)

    p = printer.Printing()
    p.pretty_print(identified_output)


class What_Object:
    def __init__(self):
        self.id = identifier.Identifier()

    def what_is_this(self, text: str) -> dict:
        """
        Returns a Python dictionary of everything that has been identified
        """
        return self.id.identify(text)


if __name__ == "__main__":
    main()



from langdetect import detect_langs
import langdetect


class LanguageDetector:
    def detect_what_lang(self, text):
        try:
            return detect_langs(text)
        except langdetect.lang_detect_exception.LangDetectException as e:
            return None



from pywhat.regex_identifier import RegexIdentifier
from pywhat.languageDetector import LanguageDetector
from pywhat.magic_numbers import FileSignatures
from pywhat.nameThatHash import Nth
import os.path


class Identifier:
    def __init__(self):
        self.regex_id = RegexIdentifier()
        self.lang_detect = LanguageDetector()
        self.file_sig = FileSignatures()
        self.name_that_hash = Nth()

    def identify(self, text: str) -> dict:
        identify_obj = {}

        magic_numbers = None
        if self.file_exists(text):
            magic_numbers = self.file_sig.open_binary_scan_magic_nums(text)
            text = self.file_sig.open_file_loc(text)
            identify_obj["File Signatures"] = magic_numbers

        if not magic_numbers:
            # If file doesn't exist, check to see if the inputted text is
            # a file in hex format
            identify_obj["File Signatures"] = self.file_sig.check_magic_nums(text)

        identify_obj["Language"] = self.lang_detect.detect_what_lang(text)
        identify_obj["Regexes"] = self.regex_id.check(text)
        # get_hashes takes a list of hashes, we split to give it a list
        identify_obj["Hashes"] = self.name_that_hash.get_hashes(text.split())

        return identify_obj

    def file_exists(self, text):
        return os.path.isfile(text)



import json
from rich.console import Console
from rich.table import Table


class Printing:
    def __init__(self):
        pass

    def pretty_print(self, text: dict):
        console = Console(highlight=False)

        languages = text["Language"]

        # calculate probability of each language
        prob_language = ""
        if languages:
            for i in languages:
                prob_language += (
                    f" [red]{i.lang}[/red] {round(i.prob * 100)}% probability"
                )

        to_out = ""
        if prob_language:
            to_out += f"[bold #D7Afff]Possible language (ISO-639-1 code):[/bold #D7Afff]{prob_language}.\n"

        if text["File Signatures"]:
            to_out += "\n"
            to_out += f"[bold #D7Afff]File Identified[/bold #D7Afff] with Magic Numbers {text['File Signatures']['ISO 8859-1']}."
            to_out += f"\n[bold #D7Afff]File Description:[/bold #D7Afff] {text['File Signatures']['Description']}."
            to_out += "\n"
        console.print(to_out)
        to_out = ""

        if text["Regexes"] and len(text["Regexes"]):
            to_out += f"\n[bold #D7Afff]Possible Identification[/bold #D7Afff]"
            table = Table(
                show_header=True, header_style="bold #D7Afff", show_lines=True
            )
            table.add_column("Matched Text")
            table.add_column("Identified as")
            table.add_column("Description")
            for i in text["Regexes"]:
                matched = i["Matched"]
                name = i["Regex Pattern"]["Name"]
                description = self.get_crypto_links(name, matched)
                if not description:
                    description = i["Regex Pattern"]["Description"]
                table.add_row(
                    matched,
                    name,
                    description,
                )
            console.print(to_out, table)
        if text["Hashes"] and len(text["Hashes"]) > 1:
            to_out = f"\n[bold #D7Afff]Hashes Identified[/bold #D7Afff]"
            table = Table(
                show_header=True, header_style="bold #D7Afff", show_lines=True
            )
            table.add_column("Matched Text")
            table.add_column("Possible Hash Type")
            table.add_column("Description")
            for hash_text in text["Hashes"].keys():
                for types in text["Hashes"][hash_text]:
                    table.add_row(
                        hash_text,
                        types["name"],
                        types["description"],
                    )
            console.print(to_out, table)

    def print_json(self, text: dict):
        return json.dumps(text, indent=4)

    def get_crypto_links(self, text, matched):
        explorers = {
            "Ethereum Wallet": "https://etherscan.io/address/",
            "Dogecoin Wallet Address": "https://dogechain.info/address/",
            "Bitcoin Wallet": "https://www.blockchain.com/btc/address/",
            "YouTube Video ID": "https://www.youtube.com/watch?v=",
        }
        if text in explorers:
            return "Click here to analyse in the browser " + explorers[text] + matched
        return None



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['pywhat']

package_data = \
{'': ['*']}

install_requires = \
['PyYAML>=5.4.1,<6.0.0',
 'click>=7.1.2,<8.0.0',
 'langdetect>=1.0.8,<2.0.0',
 'name-that-hash>=1.7.2,<2.0.0']

entry_points = \
{'console_scripts': ['pywhat = pywhat.what:main', 'what = pywhat.what:main']}

setup_kwargs = {
    'name': 'pywhat',
    'version': '0.1.11',
    'description': 'What is that thing?',
    'long_description': None,
    'author': 'Bee',
    'author_email': 'github@skerritt.blog',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)



#! /usr/bin/env python3

"""
Name That Hash: Name the hash type.
"""

import sys

if __name__ == "__main__":
    major = sys.version_info[0]
    minor = sys.version_info[1]

    python_version = (
        str(sys.version_info[0])
        + "."
        + str(sys.version_info[1])
        + "."
        + str(sys.version_info[2])
    )

    if major != 3 or (major == 3 and minor < 6):
        print(
            f"What requires Python 3.6+, you are using {python_version}. Please install a higher Python version."
        )
        sys.exit(1)

    from pywhat import what

    what.main()
